import constants from './constants.mjs';

export default new class handranker {
  rankHands(board, hand) {
    let bestHand = this.calcBestHand([...board, ...hand].map(c => this.toCard(c)));

    if(bestHand > 6000) bestHand += 2000;

    return {
      rank: bestHand,
      percentage: (9999 - bestHand) / 100,
      combination: this.toCombination(bestHand)
    };
  };

  toCombination(r) {
    if(r > 6185) return 'HighCard';
    if(r > 3325) return 'Pair';
    if(r > 2467) return 'TwoPairs';
    if(r > 1609) return 'ThreeOfAKind';
    if(r > 1599) return 'Straight';
    if(r > 322)  return 'Flush';
    if(r > 166)  return 'FullHouse';
    if(r > 10)   return 'FourOfAKind';
    return 'StraightFlush';
  };

  cactusFastRankHand(hand) {
    const [c0, c1, c2, c3, c4] = hand;
    if((c0 & c1 & c2 & c3 & c4 & 0xf000) !== 0)
      return constants.fastFlushes[(c0 | c1 | c2 | c3 | c4) >>> 16];
    const r = constants.fastUnique5[(c0 | c1 | c2 | c3 | c4) >>> 16];
    if(r)
      return r;
    let u = 0xe91aaa35 + (((c0 & 0xff) * (c1 & 0xff) * (c2 & 0xff) * (c3 & 0xff) * (c4 & 0xff)) | 0);
    u = u ^ (u >>> 16);
    u += u << 8;
    u ^= u >>> 4;
    return constants.hash[((u + (u << 2)) >>> 19) ^ (constants.hashAdjust[(u >>> 8) & 0x1ff] | 0)];
  };

  calcBestHand(hand) {
    if(hand.length === 5)
      return this.cactusFastRankHand([hand[0], hand[1], hand[2], hand[3], hand[4]])
    if(hand.length === 6) {
      const possibleHands = [
        [hand[0], hand[1], hand[2], hand[3], hand[4]],
        [hand[0], hand[1], hand[2], hand[3], hand[5]],
        [hand[0], hand[1], hand[2], hand[4], hand[5]],
        [hand[0], hand[1], hand[3], hand[4], hand[5]],
        [hand[0], hand[2], hand[3], hand[4], hand[5]],
        [hand[1], hand[2], hand[3], hand[4], hand[5]],
      ];
      const sortedHands = possibleHands.map(h => this.cactusFastRankHand(h)).sort();
      return sortedHands[0];
    }
    if(hand.length === 7) {
      let r = 0;
      let rank = 9999;
      for(let i = 0; i < 21; i++) {
        const inputHand = [
          hand[constants.t7c5[i][0]],
          hand[constants.t7c5[i][1]],
          hand[constants.t7c5[i][2]],
          hand[constants.t7c5[i][3]],
          hand[constants.t7c5[i][4]],
        ];
        r = this.cactusFastRankHand(inputHand);
        if(r < rank)
          rank = r;
      }
      return rank;
    }
    throw `Hand ranker doesn't support ${hand.length} cards`;
  };

  toCard(playingCard) {
    const rank = constants.runeToRank[playingCard[0]];
    const suit = constants.runeToSuit[playingCard[1]];
    if(!suit || rank === undefined)
      throw `Invalid playing card: ${playingCard}`;
    return ((1 << rank) << 16) | (suit << 12) | (rank << 8) | constants.PRIMES[rank];
  };
};
