import cuffeo from 'cuffeo';
import handranker from './inc/handranker.mjs';
import helper from './inc/helper.mjs';

const cfg = helper.config;
export const bot = await new cuffeo(cfg.getFull().clients);

export const env = {
  gamestate: 'not started, ' + (cfg.get('autojoin') ? 'autojoin' : 'no autojoin'),
  hand: false,
  board: false,
  odds: false,
  joined: false,
  callamount: 0,
  pot: 0,
  bank: 0,
  lastaction: false // debug purpose
};

bot.on("notice", msg => {
  if(msg.match(/^Your hand/)) {
    env.hand = helper.parseCards(msg);
    helper.debug('hand: ' + env.hand.join(', '));
  }
});

bot.on("message", async e => {
  if(e.channel !== cfg.get('channel'))
    return;
  if(e.message.startsWith('.cards ')) {
    let cards = [];
    try {
      cards = JSON.parse(e.message.slice(7));
    } catch(err) {
      return e.reply('That is not an array.');
    }

    const hand = cards.slice(0, 2);
    const board = cards.slice(2);

    let rank;

    try {
      rank = handranker.rankHands(board, hand);
    } catch(err) {
      return e.reply(err);
    }

    e.reply(JSON.stringify(rank));
    
    return;
  }
  if(e.message === `.${e.self.me.nickname} help`) {
    await e.write(`PRIVMSG ${e.user.nick} I always say hirc schmirc, available commands are:`);
    const commands = [
      '    .pj                       --   poker join (activates autojoin)',
      '    .pl                       --   poker leave (deactivates autojoin)',
      '    .bb                       --   bank balance',
      '    .bl                       --   bank loan',
      '    .d                        --   deal',
      '    .deb                      --   toggle debug',
      '    .aj                       --   toggle autojoin',
      'While playing:',
      '    .s                        --   stack',
      '    .f                        --   fold',
    ];

    for(const command of commands) {
      await e.write(`PRIVMSG ${e.user.nick} ${command}`);
    }
  }
  if(e.message.match(new RegExp(`^${e.self.me.nickname}: Your bank account`))) {
    env.bank = +e.message.match(/is: (\d+) \(/)[1];
  }
  if(e.message.startsWith(".debug")) {
    const tmp = e.message.match(/\.debug (\w+)/)?.[1];
    if(!tmp)
      return e.reply(`debug mode ${await cfg.set('debug', !cfg.get('debug')) ? '[color=green]enabled[/color]' : '[color=red]disabled[/color]'}`);
    if(!['console', 'chat'].includes(tmp))
      return e.reply(`debug mode ${await cfg.set('debug', false) ? '[color=green]enabled[/color]' : '[color=red]disabled[/color]'}`);
    return e.reply(`debug mode [color=green]enabled[/color] (${await cfg.set('debug', tmp)})`);
  }
  if(e.message === ".aj" || e.message === ".autojoin") {
    const tmp = await cfg.set('autojoin', !cfg.get('autojoin'));
    return e.reply(`autojoin ${tmp ? '[color=green]enabled[/color]' : '[color=red]disabled[/color]'}`);
  }
  if(e.message === ".pj" || (e.message === "pj" && cfg.get('autojoin')) && !env.hand && !env.joined) {
    env.joined = true;
    await cfg.set('autojoin', true);
    //return e.reply("pj");
    return e.reply(["bb", "pj"]);

  }
  if(e.message === ".pl" && !env.hand) {
    env.joined = false;
    await cfg.set('autojoin', false);
    return e.reply("pl");
  }
  if(e.message === ".bb" && !env.hand)
    return e.reply("bb");
  if(e.message === ".bl" && !env.hand)
    return e.reply("bank loan");
  if(e.message === ".s" && env.hand)
    return e.reply("s");
  if(e.message === ".f")
    return e.reply("f");
  if(e.message === ".d" && !env.hand)
    return e.reply("d");
  if(e.message === ".env") {
    return helper.debug(env, 5);
  }

  if(e.user.nick !== cfg.get('pokerbot') || !env.hand)
    return;
  
  // tracker
  if(e.message.includes('raises the pot')) {
    env.callamount = +e.message.match(/raises the pot by (\d+)/)[1];
    env.pot += env.callamount;
    if(e.message.includes(e.self.me.nickname)) {
      env.bank -= env.callamount;
      helper.debug(`bank: ${env.bank}`);
    }
    return;
  }
  if(e.message.includes('calls')) {
    env.callamount = +e.message.match(/calls (\d+)\./)[1];
    env.pot += env.callamount;
    if(e.message.includes(e.self.me.nickname)) {
      env.bank -= env.callamount;
      helper.debug(`bank: ${env.bank}`);
    }
    return;
  }
  if(e.message.endsWith('blind).')) { // blinds
    env.callamount = +e.message.match(/pays (\d+) \(/)[1];
    env.pot += env.callamount;

    const oldstate = env.gamestate;
    env.gamestate = 'preflop';
    if(oldstate !== env.gamestate)
      helper.debug(`${oldstate} -> ${env.gamestate}`);

    if(e.message.includes(e.self.me.nickname)) {
      env.bank -= env.callamount;
      helper.debug(`bank: ${env.bank}`);
    }
    return;
  }

  // bot is broke :(
  if(e.message.includes('You don\'t have enough money')) {
    e.reply(['fugggggg', 'f']);
    if(env.bank < 100) {
      e.reply('bank loan');
    }
    return;
  }
  
  // bot's turn
  if(e.message.match(new RegExp(`Current player: ${e.self.me.nickname}( |$)`))) {
    let action = 'c'; // default
    let debug = false;
    if(env.gamestate === 'preflop' || !env.odds) { // preflop
      env.lastaction = action;
      helper.debug('preflop, (checkcall) :(', 3);
      return e.reply(action); // checkcall
    }

    if(e.message.includes('to call')) { // callphase
      env.callamount = +e.message.match(/\((\d+) to call\)/)[1];

      if(env.callamount > env.bank) // not enough money lol
        return e.reply(['huan!', 'f']);
      
      if(env.odds <= 40) {
        if(helper.rand(20) === 1 && env.callamount < (helper.rand(2, 6) * 10)) { // bad hand, call anyway
          action = 'call';
          debug = `bad hand, call anyway (${env.odds}, callphase, 5%)`;
        }
        else { // bad hand, fold
          action = 'fold';
          debug = `bad hand (${env.odds}, callphase, 95%)`;
        }
      }
      else if(env.odds > 40 && env.odds <= 50) { // decent hand, raise
        if(helper.rand(10) === 1) { // 10%
          const toRaise = env.callamount + helper.rand(5) * 10;
          if(toRaise > env.bank) {
            action = 'raise ' + toRaise.toString();
            debug = `decent hand (${env.odds}, callphase, 10%)`;
          }
        }
      }
      else if(env.odds > 50 && env.odds <= 70) { // good hand lol
        if(helper.rand(3) === 1) { // 33%
          const toRaise = env.callamount + helper.rand(6) * 10;
          if(toRaise > env.bank) {
            action = 'raise ' + toRaise.toString();
            debug = `good hand lol (${env.odds}, callphase, 33%)`;
          }
        }
      }
      else if(env.odds > 70) { // fuck them all
        const toRaise = env.callamount + helper.rand(7) * 10;
        if(toRaise > env.bank) {
          action = 'raise ' + toRaise.toString();
          debug = `fuck them all (${env.odds}, callphase)`;
        }
      }
    }
    else { // checkphase
      if(env.odds > 55 && env.odds <= 70) { // decend hand, raise
        if(helper.rand(5) === 1) { // 20%
          const toRaise = env.callamount + helper.rand(5) * 10;
          if(toRaise > env.bank) {
            action = 'raise ' + toRaise.toString();
            debug = `decent hand (${env.odds}, checkphase, 20%)`;
          }
        }
      }
      else if(env.odds > 70 && env.odds <= 85) { // good hand lol
        if(helper.rand(2) === 1) { // 50%
          const toRaise = env.callamount + helper.rand(6) * 10;
          if(toRaise > env.bank) {
            action = 'raise ' + toRaise.toString();
            debug = `good hand lol (${env.odds}, callphase, 50%)`;
          }
        }
      }
      else if(env.odds > 85) { // fuck them all
        const toRaise = env.callamount + helper.rand(7) * 10;
        if(toRaise > env.bank) {
          action = 'raise ' + toRaise.toString();
          debug = `fuck them all (${env.odds}, callphase)`;
        }
      }
    }

    if(action) {
      env.lastaction = action;
      helper.debug(debug ? debug : 'check lol', 3);
      return e.reply(action);
    }
  }

  // gamestate & board changes
  if(e.message.match(/^(Flop|Turn|River)/)) {
    env.board = helper.parseCards(e.message);
    const oldstate = env.gamestate;
    env.gamestate = e.message.match(/(\w+): /)[1].toLowerCase();

    helper.debug(`${oldstate} -> ${env.gamestate}`, 2);

    const rank = handranker.rankHands(env.board, env.hand);
    env.odds = rank.percentage;
  }

  // end of game
  if(e.message.includes('wins the pot') || e.message.includes('Split pot')) {
    if(e.message.includes(e.self.me.nickname)) {
      if(e.message.includes('Split pot')) {
        const tmp = e.message.match(/size (\d+) .*: (.*) \(/);
        env.bank += ~~(tmp[1] / tmp[2].split(', ').length);
      }
      else {
        env.bank += +e.message.match(/pot(: | of size )(\d+)/)[2];
      }

      helper.debug(`bank: ${env.bank}`);
    }

    const oldstate = env.gamestate;
    env.gamestate = 'not started, ' + (cfg.get('autojoin') ? 'autojoin' : 'no autojoin');
    helper.debug(`${oldstate} -> ${env.gamestate}`, 2);

    env.hand = false;
    env.board = false;
    env.odds = false;
    env.joined = false;
    env.pot = 0;
    env.lastaction = false;
    return;
  }
});
